!******************************************************************************!
!                                                                              !
!    class_colorflow.f90                                                       !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2020 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   class_colorflow.f90                            |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 07/2020                              |            !
!        |       | UFO version: 1.0                               |            !
!        |       | Generation date: 2020-08-04 12:55             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module class_colourflow
  use input_mdl, only: error_mdl
  use class_vertices
  implicit none
  contains
    subroutine compute_colour_correlation_mdl(colorflowIds, csIn, U1GluonIn,   &
    csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE, ncpower,    &
    n_In)
      integer, dimension(:),     intent(inout) :: select_branch
      integer, dimension(:,-1:), intent(in)    :: csIn
      integer, allocatable,      intent(inout) :: csOut(:,:)
      logical, dimension(:),     intent(in)    :: U1GluonIn
      logical,                   intent(inout) :: U1GluonOut
      integer, dimension(-1:0),  intent(in)    :: csLast
      logical,                   intent(in)    :: last
      integer,                   intent(in)    :: pr,legsE,lp,n_In
      integer, dimension(0:),    intent(in)    :: colorflowIds
      integer, dimension(:),     intent(inout) :: ncpower
      integer :: nbranches
      nbranches = size(select_branch)
      select case(n_In)
      case(1)
        call compute_colour_correlation2(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(2)
        call compute_colour_correlation3(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(3)
        call compute_colour_correlation4(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(4)
        call compute_colour_correlation5(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(5)
        call compute_colour_correlation6(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(6)
        call compute_colour_correlation7(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case(7)
        call compute_colour_correlation8(nbranches, colorflowIds, csIn,        &
        U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp,     &
        legsE, ncpower)
      case default
        call error_mdl("No colourflow for `n_In`< 1 | `n_In`> 7.", where="     &
        compute_colour_correlation_mdl")
      end select 
    end subroutine compute_colour_correlation_mdl

    subroutine compute_colour_correlation2(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
!       d^1_2 d^2_1
        case(73)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(1)
!       d^2_1
        case(461)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_2
        case(74)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2
        case(72)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation2')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation2

    subroutine compute_colour_correlation3(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
!       d^1_3 d^3_2
        case(10)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_3 d^3_2
        case(469)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^3_2
        case(103)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^3_3
        case(11)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_1 d^3_3
        case(99)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_3
        case(462)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3
        case(464)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
        case(9)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^3_1
        case(97)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^2_2 d^3_1
        case(468)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^3_1
        case(22)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(1)
!       d^3_2
        case(463)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3
        case(96)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^3_3
        case(106)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_3 d^3_2
        case(105)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(2)
!       d^1_1 d^2_3
        case(101)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_3 d^3_1
        case(100)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1
        case(16)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_1
        case(466)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^3_3
        case(23)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2 d^2_1 d^3_3
        case(470)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_1
        case(467)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_2
        case(17)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_1 d^3_2
        case(98)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2
        case(465)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_2
        case(1)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_2
        case(95)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^3_1
        case(104)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_1
        case(0)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1
        case(102)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_2 d^3_3
        case(471)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation3')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation3

    subroutine compute_colour_correlation4(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
!       d^1_1 d^4_2
        case(487)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^4_3
        case(491)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^3_4
        case(484)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_4 d^4_1
        case(506)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_1 d^4_3
        case(416)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_1 d^4_3
        case(500)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3
        case(520)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^3_3
        case(89)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^3_3
        case(87)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_1
        case(475)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_2 d^4_4
        case(533)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_1 d^4_2
        case(496)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_3 d^4_2
        case(517)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^3_1
        case(488)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_2 d^3_4 d^4_1
        case(544)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_2 d^3_4 d^4_3
        case(534)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^2_3 d^3_1
        case(28)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_2 d^3_3 d^4_4
        case(546)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3 d^4_4
        case(531)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2 d^2_4 d^3_3 d^4_1
        case(545)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^3_1
        case(474)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^4_2
        case(510)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_4
        case(495)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^2_1
        case(492)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_2
        case(521)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_3 d^4_1
        case(501)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_3 d^4_4
        case(515)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^3_2 d^4_4
        case(519)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_2 d^4_1
        case(497)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^3_2
        case(524)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_2 d^4_4
        case(543)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2 d^2_1 d^3_3 d^4_4
        case(551)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^3_4 d^4_3
        case(94)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(3)
!       d^3_3 d^4_4
        case(93)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_4 d^2_1 d^3_2 d^4_3
        case(5)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_3 d^4_2
        case(532)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_3 d^3_4
        case(512)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_1
        case(473)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_1 d^3_3
        case(505)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1
        case(14)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_2 d^4_3
        case(516)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_3 d^3_2 d^4_4
        case(547)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_4 d^2_2 d^3_1 d^4_3
        case(536)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4 d^2_2 d^3_3 d^4_1
        case(552)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^2_2
        case(15)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_1 d^3_2
        case(498)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2
        case(483)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_2
        case(92)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(2)
!       d^2_2 d^3_4
        case(509)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^2_3 d^3_1 d^4_4
        case(538)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^4_3
        case(478)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^4_4
        case(91)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3 d^2_4 d^3_1 d^4_2
        case(540)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_4 d^3_2 d^4_3
        case(541)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_1 d^4_4
        case(507)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2 d^2_1 d^3_4 d^4_3
        case(539)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_3 d^3_2 d^4_1
        case(537)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_2
        case(523)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4 d^3_1
        case(485)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^3_1
        case(499)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_2
        case(508)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_2
        case(29)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_4 d^4_2
        case(145)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^3_2
        case(489)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_3 d^3_4 d^4_2
        case(542)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^3_4 d^4_1
        case(415)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_2 d^3_4 d^4_3
        case(548)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4 d^2_3 d^3_1 d^4_2
        case(3)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^4_1
        case(53)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4
        case(522)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_3
        case(530)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_1
        case(88)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^4_4
        case(21)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2 d^3_3
        case(525)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_2 d^4_3
        case(146)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^3_1 d^4_3
        case(4)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4
        case(472)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_2 d^3_1 d^4_4
        case(550)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3
        case(481)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_2
        case(477)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^4_2
        case(52)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_4 d^3_2 d^4_1
        case(2)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_3
        case(513)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^3_1 d^4_4
        case(503)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_3 d^3_2
        case(90)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_1
        case(502)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_2
        case(476)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_1
        case(20)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(1)
!       d^1_1 d^2_3
        case(493)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3
        case(479)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_1
        case(504)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_3
        case(529)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^3_4
        case(528)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_2
        case(518)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
        case(8)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^4_1
        case(486)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4
        case(480)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^4_1
        case(490)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_2
        case(526)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^4_3
        case(511)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4 d^2_1
        case(494)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_3
        case(514)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_4 d^3_3 d^4_2
        case(549)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4 d^2_1 d^3_3 d^4_2
        case(535)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4
        case(482)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_4 d^4_1
        case(7)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^4_4
        case(527)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3 d^2_1 d^3_4 d^4_2
        case(6)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation4')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation4

    subroutine compute_colour_correlation5(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
!       d^2_4 d^5_2
        case(664)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_3 d^3_1 d^5_2
        case(67)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^4_5
        case(661)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(4,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^3_5 d^4_4 d^5_3
        case(674)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_2 d^3_1
        case(599)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_3 d^3_5 d^5_2
        case(698)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^5_1
        case(572)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^4_2 d^5_5
        case(717)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(4,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(csIn(4,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_4 d^2_1 d^4_2
        case(244)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^3_5
        case(557)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4 d^3_3
        case(611)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^3_4
        case(667)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_2
        case(623)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^5_4
        case(648)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_3
        case(644)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_3 d^5_5
        case(631)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(4,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_5 d^3_2
        case(666)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_1 d^4_5 d^5_4
        case(725)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_4 d^4_1 d^5_2
        case(76)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_5 d^5_1
        case(299)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_3 d^3_1 d^4_2
        case(43)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^4_3 d^5_4
        case(252)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_4 d^4_2
        case(417)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_3
        case(606)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_5 d^4_3 d^5_1
        case(85)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_1
        case(135)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^4_2
        case(662)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^4_4
        case(138)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_5 d^5_1
        case(297)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_1 d^4_4
        case(589)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_3 d^5_4
        case(630)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_3 d^4_5
        case(627)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(4,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^2_4 d^3_1 d^4_3
        case(44)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_5
        case(646)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^3_3 d^4_1
        case(690)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_2 d^4_5 d^5_4
        case(678)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_2
        case(621)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_2 d^5_3
        case(650)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^5_4
        case(555)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(4,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_2 d^4_3
        case(671)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^4_1 d^5_5
        case(716)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(4,0)
            csTmp(csIn(4,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_5 d^3_2 d^4_3 d^5_4
        case(253)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_1
        case(560)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_2 d^3_4 d^4_1
        case(693)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_2
        case(668)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^4_4
        case(452)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_2
        case(27)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^5_4
        case(663)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(4,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^2_4 d^3_2 d^4_1
        case(42)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_1 d^4_4
        case(581)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_2
        case(449)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^4_1 d^5_4
        case(300)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_3 d^5_5
        case(142)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3 d^3_4 d^4_1
        case(245)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_5 d^4_2 d^5_4
        case(257)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^5_2
        case(139)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(2)
!       d^4_2 d^5_4
        case(658)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_1
        case(137)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^3_2
        case(575)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^4_1
        case(243)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4
        case(622)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^5_5
        case(603)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(4,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_5 d^3_4 d^4_2 d^5_3
        case(254)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_4 d^4_5 d^5_2
        case(709)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(4,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^2_1 d^3_2 d^4_4
        case(695)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_1 d^4_3
        case(246)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_5 d^3_2 d^5_3
        case(707)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_4 d^3_2 d^4_3
        case(418)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_4 d^4_5 d^5_2
        case(256)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^5_1
        case(18)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(1)
!       d^4_4 d^5_1
        case(577)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^2_5
        case(565)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^4_1
        case(574)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_5 d^5_4
        case(143)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(4)
!       d^1_5 d^5_4
        case(604)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_5 d^4_3 d^5_2
        case(255)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_4 d^4_2 d^5_1
        case(713)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_3 d^3_4 d^4_2
        case(687)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1
        case(568)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^5_5
        case(607)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_3 d^5_5
        case(641)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_4 d^2_1 d^3_3 d^4_2
        case(688)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^5_4
        case(636)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_1 d^4_3 d^5_4
        case(86)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_1 d^5_5
        case(591)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_2 d^2_3 d^3_4 d^4_1
        case(47)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^5_1
        case(556)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1 d^4_5 d^5_4
        case(714)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^4_4
        case(601)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^2_1 d^3_4 d^4_2
        case(46)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_3
        case(569)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^4_4
        case(633)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_5 d^2_1 d^4_2 d^5_4
        case(80)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_3
        case(453)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_3 d^5_2
        case(649)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^3_5 d^5_1
        case(584)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_3 d^4_2
        case(651)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_1
        case(594)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_5 d^3_3 d^5_1
        case(701)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^2_1 d^3_2 d^5_5
        case(706)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^3_5 d^5_3
        case(141)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
          U1GluonOut = U1GluonIn(3)
!       d^1_2 d^2_5 d^5_1
        case(50)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_3
        case(637)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^3_4
        case(612)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_2 d^3_5 d^5_1
        case(704)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^4_5 d^5_2
        case(660)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^4_5 d^5_2
        case(77)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^5_3
        case(642)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^5_2
        case(616)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_1 d^4_3 d^5_5
        case(728)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(4,0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            csTmp(csIn(4,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3 d^4_1
        case(564)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_5
        case(634)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_5 d^5_3
        case(251)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_1 d^3_3 d^5_2
        case(699)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4 d^4_2
        case(614)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_3 d^4_5 d^5_1
        case(726)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(4,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(1,-1), bin_color) = csIn(4,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_2 d^3_3
        case(450)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_5
        case(602)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_1
        case(558)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_1 d^3_2 d^5_3
        case(68)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_5 d^5_2
        case(71)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^3_3
        case(617)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^4_2 d^5_4
        case(250)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_5 d^3_1 d^5_2
        case(700)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_3 d^4_5 d^5_2
        case(686)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(4,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(4,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^2_5 d^4_1 d^5_4
        case(78)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^5_2
        case(571)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^3_3
        case(136)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^5_2
        case(654)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1
        case(12)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^3_4 d^4_5 d^5_3
        case(720)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(4,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_5 d^2_1
        case(566)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_5 d^4_2 d^5_1
        case(79)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_1 d^5_4
        case(578)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_4 d^4_3 d^5_2
        case(680)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_5 d^4_4 d^5_2
        case(679)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4 d^3_5 d^4_1 d^5_3
        case(722)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^3_2 d^4_3
        case(45)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_5 d^4_2 d^5_3
        case(681)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_1 d^5_5
        case(583)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_1 d^4_3
        case(563)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_2 d^4_5 d^5_1
        case(715)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(4,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(1,-1), bin_color) = csIn(4,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^4_2
        case(573)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_3 d^4_1 d^5_4
        case(730)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(4,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            csTmp(csIn(4,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^2_3 d^3_5 d^5_1
        case(70)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^3_4
        case(559)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_1 d^5_5
        case(579)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(4,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^4_4 d^5_2
        case(657)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^4_4
        case(605)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1 d^3_5 d^5_3
        case(703)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^5_5
        case(647)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(4,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_2 d^5_1
        case(595)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_2 d^5_5
        case(140)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^4_4 d^5_5
        case(144)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_1 d^4_5
        case(553)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(4,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^3_4 d^5_3
        case(626)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_4
        case(567)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_2 d^5_3
        case(248)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^3_4 d^4_5 d^5_3
        case(685)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(4,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_4 d^2_5 d^4_1 d^5_2
        case(711)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_2 d^4_5 d^5_3
        case(258)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_1
        case(582)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_3 d^4_5 d^5_4
        case(673)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(4,-1)
            csTmp(0,bin_color) = csIn(4,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^2_5 d^4_2 d^5_4
        case(718)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(4,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(4,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^4_2 d^5_5
        case(659)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(4,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_2 d^5_3
        case(669)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^3_5
        case(610)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_2 d^4_4 d^5_3
        case(684)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^5_3
        case(561)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^2_4 d^4_5 d^5_1
        case(75)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_5 d^3_2 d^5_1
        case(69)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_5 d^4_4 d^5_1
        case(712)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_1 d^4_2
        case(598)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^3_5
        case(665)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_4 d^3_3
        case(639)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_4 d^4_5 d^5_1
        case(81)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_5 d^5_3
        case(632)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_2 d^4_1 d^5_4
        case(719)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(4,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(4,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^2_5 d^3_1 d^5_3
        case(66)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_4 d^4_1 d^5_5
        case(727)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(4,0)
            csTmp(csIn(4,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^3_3 d^4_4
        case(454)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^3_1
        case(576)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_5
        case(620)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_1 d^4_4 d^5_3
        case(721)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^2_3 d^3_1
        case(26)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_3 d^3_2 d^5_1
        case(702)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^5_1
        case(592)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^3_5 d^4_3 d^5_4
        case(729)
          if (U1GluonIn(1)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(4,0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(csIn(4,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^3_1 d^5_3
        case(586)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_5
        case(638)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_4 d^4_3 d^5_1
        case(724)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^5_3
        case(608)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_2
        case(619)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_3 d^4_2
        case(672)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3
        case(624)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^5_5
        case(615)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3 d^2_1
        case(570)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^4_1
        case(554)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_1 d^4_3
        case(588)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_2 d^4_3
        case(652)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_1 d^5_5
        case(705)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_5 d^3_1 d^5_3
        case(298)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_1 d^4_5 d^5_3
        case(83)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_3 d^4_1
        case(587)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^5_2
        case(670)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^4_4
        case(643)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_2 d^4_4
        case(655)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^4_1
        case(597)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_2 d^3_1 d^4_3
        case(697)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_5 d^5_2
        case(249)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_2
        case(618)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_5 d^4_1 d^5_4
        case(84)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_5 d^5_2
        case(247)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_3 d^5_5
        case(675)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(3,-1), bin_color) = csIn(4,0)
            csTmp(csIn(4,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_1 d^2_2
        case(13)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_1 d^3_2
        case(600)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_3
        case(609)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_5 d^3_4 d^4_1 d^5_3
        case(82)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_5 d^5_1
        case(580)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^5_5
        case(635)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(3,-1), bin_color) = csIn(4,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_4 d^3_2 d^4_3 d^5_5
        case(682)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(4,0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            csTmp(csIn(4,-1), bin_color) = csIn(3,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_3 d^3_4
        case(640)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_2
        case(451)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^4_4
        case(613)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_1 d^5_2
        case(51)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_1
        case(590)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_3 d^5_4
        case(625)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(4,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_5 d^4_4
        case(645)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = 0
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_2 d^2_3 d^3_1 d^4_4
        case(694)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_5 d^4_4 d^5_1
        case(723)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(1,-1), bin_color) = csIn(3,0)
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_5 d^2_1 d^4_4 d^5_2
        case(710)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(2,0)
            csTmp(csIn(2,-1), bin_color) = csIn(1,0)
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_3 d^2_4 d^3_1 d^4_2
        case(689)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_1 d^2_4 d^3_2 d^4_3
        case(696)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(1,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_4 d^4_2 d^5_5
        case(683)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(2,-1), bin_color) = csIn(3,0)
            csTmp(csIn(3,-1), bin_color) = csIn(4,0)
            csTmp(csIn(4,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_1 d^3_3
        case(593)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_1 d^3_4 d^4_3
        case(692)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_3 d^3_2 d^4_1
        case(691)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_2
        case(656)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_4 d^5_3
        case(629)
          if (U1GluonIn(4)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(4,-1), bin_color) = csIn(4,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^3_2 d^5_5
        case(653)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(3,-1), bin_color) = csIn(2,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^2_5 d^3_3 d^4_2 d^5_4
        case(676)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = csIn(2,-1)
            csTmp(0,bin_color) = csIn(4,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            csTmp(csIn(4,-1), bin_color) = csIn(2,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^1_1 d^5_5
        case(19)
          if (last) then
            csTmp(csLast(-1), bin_color) = csLast(0)
            csTmp(csIn(1,-1), bin_color) = csIn(1,0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
          U1GluonOut = .true.
!       d^1_3 d^5_1
        case(562)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_2 d^3_1 d^5_3
        case(708)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(1,-1)
            csTmp(0,bin_color) = csIn(3,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(3,-1), bin_color) = csIn(1,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^2_1 d^5_2
        case(596)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^4_3
        case(628)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = 0
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_2 d^3_5 d^4_3 d^5_4
        case(677)
          if (U1GluonIn(2)) then
            csTmp(-1,bin_color) = csIn(3,-1)
            csTmp(0,bin_color) = csIn(4,0)
            csTmp(csIn(2,-1), bin_color) = csIn(2,0)
            csTmp(csIn(4,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
!       d^3_3 d^5_1
        case(585)
          if (U1GluonIn(3)) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = csIn(1,0)
            csTmp(csIn(3,-1), bin_color) = csIn(3,0)
            if (last) then
              csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
              csTmp(-1,bin_color) = 0
              csTmp(0,bin_color) = 0
            end if 
            select_branch(bin_color) = cf + 1
          else
            bin_color = bin_color - 1
          end if 
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation5')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation5

    subroutine compute_colour_correlation6(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
!       d^4_5 d^5_6 d^6_4
        case(127)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_1
        case(24)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_5 d^5_1
        case(148)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_5 d^5_6 d^6_1
        case(290)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_2
        case(25)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_5 d^3_2 d^5_1
        case(152)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^3_1 d^4_3
        case(38)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_6 d^5_1 d^6_5
        case(294)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_6 d^4_5 d^5_3 d^6_4
        case(183)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_2 d^5_6 d^6_3
        case(176)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_2 d^4_5 d^5_3
        case(457)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_1 d^5_3
        case(114)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_6 d^6_1
        case(130)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_6 d^6_2
        case(117)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^4_6 d^5_4 d^6_2
        case(180)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^4_3 d^5_6 d^6_4
        case(188)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_6 d^6_1
        case(63)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^4_6 d^6_2
        case(268)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^4_2 d^5_4 d^6_5
        case(179)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^4_2 d^6_4
        case(120)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^4_5 d^5_1 d^6_4
        case(291)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_5 d^5_1 d^6_2
        case(273)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_6 d^4_3 d^6_4
        case(124)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_1 d^5_2
        case(110)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_4 d^4_1 d^6_2
        case(267)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^3_1 d^5_3 d^6_5
        case(287)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^4_5 d^5_2
        case(153)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_6 d^4_3 d^6_2
        case(168)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^3_1 d^6_3
        case(129)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_3 d^3_1 d^6_2
        case(65)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^4_1 d^6_4
        case(131)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_1 d^4_3
        case(112)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_4 d^3_2 d^4_1
        case(36)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_1 d^5_6 d^6_2
        case(274)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^4_1 d^5_4 d^6_5
        case(293)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_6 d^6_2
        case(119)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^3_5 d^5_2 d^6_3
        case(171)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_4 d^4_2
        case(259)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_5 d^5_1
        case(115)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^4_3 d^5_4
        case(266)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^4_5 d^5_2 d^6_4
        case(177)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^5_1 d^6_5
        case(133)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_5 d^5_3
        case(265)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_5 d^4_3 d^5_2
        case(460)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_6 d^6_1
        case(132)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_6 d^5_3 d^6_5
        case(187)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^4_1 d^5_6 d^6_4
        case(292)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_6 d^5_3 d^6_2
        case(174)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_6 d^3_2 d^6_1
        case(62)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_4 d^4_5 d^5_2
        case(455)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_6 d^4_1 d^6_4
        case(270)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_4 d^4_2 d^5_3
        case(459)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_6 d^5_2 d^6_5
        case(181)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_5 d^5_2
        case(263)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^4_2 d^5_4
        case(264)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_5 d^5_6 d^6_3
        case(184)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_2 d^4_6 d^6_3
        case(170)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^5_6 d^6_1
        case(134)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_4 d^4_5 d^5_1
        case(160)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^3_4 d^4_2 d^6_3
        case(165)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^3_5 d^5_1 d^6_3
        case(285)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_4 d^4_1
        case(111)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^3_2 d^4_3
        case(39)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_4 d^4_1 d^5_2
        case(157)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_4 d^4_6 d^6_2
        case(166)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_6 d^5_3 d^6_5
        case(126)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_6 d^5_2 d^6_5
        case(175)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^4_1
        case(107)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_1 d^4_3 d^5_4
        case(161)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_6 d^4_1 d^6_4
        case(282)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_5 d^4_3 d^5_1
        case(164)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_4 d^4_1
        case(41)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_1 d^6_2
        case(48)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_5 d^4_1 d^5_4
        case(156)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_2 d^5_3
        case(262)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_6 d^5_3 d^6_1
        case(289)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_6 d^4_3 d^6_1
        case(283)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_6 d^6_3
        case(123)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_6 d^4_2 d^6_4
        case(169)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^3_2 d^4_3 d^6_4
        case(167)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_5 d^4_1 d^5_4
        case(162)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_5 d^5_6 d^6_1
        case(296)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_4 d^4_1 d^5_3
        case(163)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^3_2 d^5_3 d^6_5
        case(173)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_1 d^3_2 d^5_3
        case(149)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_1 d^4_2 d^6_4
        case(269)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_5 d^5_2
        case(147)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_1 d^5_6 d^6_3
        case(286)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_5 d^4_2 d^5_1
        case(158)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_6 d^4_2 d^6_1
        case(271)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^5_2 d^6_5
        case(122)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_6 d^6_2
        case(64)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_5 d^5_1
        case(109)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^4_6 d^5_4 d^6_3
        case(186)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^4_6 d^6_1
        case(272)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_6 d^5_4 d^6_5
        case(128)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^3_2 d^6_3
        case(118)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_3 d^3_1 d^5_2
        case(151)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_1 d^5_2 d^6_5
        case(275)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_5 d^5_6 d^6_2
        case(178)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_6 d^4_3 d^5_4 d^6_5
        case(185)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_5 d^3_1 d^5_3
        case(150)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^3_1 d^4_3 d^6_4
        case(281)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_3 d^3_1 d^4_2
        case(37)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_5 d^5_2
        case(261)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^5_6 d^6_2
        case(121)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_5 d^5_6 d^6_1
        case(278)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_4 d^4_6 d^6_1
        case(284)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_4 d^4_2
        case(40)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^4_5 d^5_1
        case(154)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_1 d^4_2 d^5_4
        case(155)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_6 d^5_1 d^6_5
        case(288)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^4_6 d^5_4 d^6_1
        case(295)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^5_6 d^6_3
        case(125)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_1 d^4_5 d^5_3
        case(159)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^3_4 d^4_1 d^6_3
        case(279)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_1 d^3_2 d^6_3
        case(60)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_2 d^4_3 d^5_4
        case(456)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^4_2
        case(108)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_6 d^6_1
        case(49)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_6 d^5_1 d^6_5
        case(276)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^4_1 d^5_4
        case(116)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_5 d^4_2 d^5_4
        case(458)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_2 d^4_3
        case(260)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_6 d^3_1 d^6_3
        case(61)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_6 d^5_2 d^6_1
        case(277)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_5 d^5_6 d^6_2
        case(172)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_1 d^4_6 d^6_3
        case(280)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_5 d^5_1
        case(113)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^4_2 d^5_6 d^6_4
        case(182)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation6')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation6

    subroutine compute_colour_correlation7(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
!       d^2_6 d^3_2 d^4_3 d^6_4
        case(430)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_5 d^5_7 d^7_1
        case(198)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_7 d^6_1 d^7_6
        case(233)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^3_6 d^6_1 d^7_3
        case(223)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_1 d^5_7 d^7_2
        case(200)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_5 d^5_6 d^6_1
        case(351)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_1 d^6_7 d^7_2
        case(206)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^2_3 d^3_1 d^7_2
        case(54)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_4 d^4_1 d^5_3
        case(336)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^2_1 d^4_2 d^7_4
        case(189)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_5 d^4_1 d^5_4
        case(332)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_1 d^3_2 d^5_3
        case(305)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_1 d^4_2 d^6_4
        case(323)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_5 d^5_2
        case(304)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^4_6 d^6_2 d^7_4
        case(382)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_6 d^6_7 d^7_1
        case(204)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^3_6 d^6_2 d^7_3
        case(370)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^4_7 d^7_2
        case(194)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^4_2 d^5_4 d^7_5
        case(377)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^4_7 d^6_4 d^7_1
        case(232)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^3_5 d^5_1 d^6_3
        case(348)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^4_1 d^6_4 d^7_6
        case(231)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_7 d^5_6 d^6_3 d^7_5
        case(406)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_5 d^4_2 d^5_1
        case(313)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_7 d^5_3 d^6_5 d^7_6
        case(407)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_7 d^4_2 d^7_4
        case(360)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_6 d^6_2
        case(310)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^3_4 d^4_1 d^7_3
        case(211)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^3_5 d^5_2 d^6_3
        case(434)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_7 d^7_2
        case(55)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^2_6 d^6_1 d^7_2
        case(205)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_5 d^5_7 d^7_1
        case(216)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_6 d^3_2 d^6_1
        case(307)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^4_7 d^5_4 d^7_2
        case(374)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^3_5 d^5_2 d^7_3
        case(364)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_3 d^3_1 d^5_2
        case(306)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^5_7 d^6_5 d^7_1
        case(238)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_1 d^5_2 d^6_5
        case(329)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_5 d^5_6 d^6_2
        case(438)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_7 d^5_6 d^6_4 d^7_5
        case(412)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^4_7 d^5_4 d^7_3
        case(392)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_5 d^3_1 d^5_3
        case(302)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^3_1 d^4_3 d^7_4
        case(207)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^4_1 d^5_4 d^6_5
        case(353)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^3_1 d^6_7 d^7_3
        case(224)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^3_2 d^4_3
        case(33)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_6 d^5_1 d^6_5
        case(326)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_7 d^4_6 d^6_3 d^7_4
        case(400)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_7 d^5_3 d^7_2
        case(362)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^3_1 d^4_3 d^6_4
        case(341)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_3 d^3_1 d^4_2
        case(31)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_2 d^4_6 d^6_3
        case(427)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_5 d^5_6 d^6_1
        case(327)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^5_1 d^6_7 d^7_5
        case(242)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_7 d^5_2 d^7_5
        case(378)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^4_2 d^5_7 d^7_4
        case(373)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^3_2 d^5_3 d^6_5
        case(436)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_4 d^4_6 d^6_1
        case(339)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_1 d^3_4 d^4_2
        case(34)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_6 d^6_7 d^7_1
        case(222)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_7 d^5_1 d^7_5
        case(197)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_6 d^5_1 d^6_5
        case(344)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^4_6 d^5_4 d^6_1
        case(349)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_1 d^4_5 d^5_3
        case(334)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_5 d^5_6 d^6_7 d^7_4
        case(411)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^3_4 d^4_1 d^6_3
        case(342)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^4_6 d^6_1 d^7_4
        case(235)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_7 d^6_2 d^7_6
        case(384)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_1 d^3_2 d^6_3
        case(311)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^3_5 d^5_1 d^7_3
        case(217)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^5_6 d^6_7 d^7_3
        case(405)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_7 d^4_1 d^7_4
        case(191)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^5_6 d^6_1 d^7_5
        case(241)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_7 d^6_2 d^7_6
        case(372)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^4_7 d^5_4 d^7_1
        case(226)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_5 d^5_7 d^7_2
        case(375)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_6 d^4_3 d^6_7 d^7_4
        case(397)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^3_1 d^6_3 d^7_6
        case(219)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_5 d^4_2 d^5_4
        case(419)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_6 d^5_3 d^6_7 d^7_5
        case(403)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^2_1 d^5_2 d^7_5
        case(195)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^2_5 d^5_1 d^7_2
        case(199)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_7 d^5_4 d^6_5 d^7_6
        case(413)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_6 d^4_7 d^6_4 d^7_3
        case(398)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_7 d^5_1 d^7_5
        case(227)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_6 d^3_1 d^6_3
        case(308)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_6 d^5_2 d^6_1
        case(325)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_5 d^5_6 d^6_2
        case(432)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_1 d^4_6 d^6_3
        case(340)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^3_1 d^5_3 d^7_5
        case(213)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_6 d^6_7 d^7_2
        case(381)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^3_2 d^6_3 d^7_6
        case(371)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^4_2 d^5_6 d^6_4
        case(439)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_7 d^6_3 d^7_6
        case(402)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_1 d^4_7 d^7_3
        case(212)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^5_2 d^6_5 d^7_6
        case(389)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_2 d^5_7 d^7_3
        case(361)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_7 d^5_3 d^7_1
        case(214)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_5 d^5_6 d^6_1
        case(345)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^4_7 d^6_4 d^7_2
        case(380)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^2_1 d^6_2 d^7_6
        case(201)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^5_6 d^6_7 d^7_1
        case(240)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^5_1 d^6_5 d^7_6
        case(237)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^5_7 d^6_2 d^7_6
        case(390)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^4_2 d^6_4 d^7_6
        case(383)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^4_1 d^6_7 d^7_4
        case(236)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_6 d^4_5 d^5_3 d^6_4
        case(446)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_2 d^5_6 d^6_3
        case(433)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_2 d^4_5 d^5_3
        case(421)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_6 d^6_7 d^7_3
        case(399)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_5 d^5_7 d^7_1
        case(228)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_5 d^5_7 d^7_3
        case(393)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_7 d^4_5 d^5_3 d^7_4
        case(394)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_7 d^3_1 d^7_3
        case(57)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_6 d^4_3 d^5_4 d^6_5
        case(448)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^3_7 d^6_3 d^7_2
        case(368)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_7 d^4_3 d^5_4 d^7_5
        case(395)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^4_6 d^5_4 d^6_2
        case(441)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^3_4 d^4_2 d^7_3
        case(358)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^5_6 d^6_7 d^7_2
        case(387)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_6 d^6_1
        case(309)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^4_6 d^6_2
        case(322)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_6 d^5_3 d^6_1
        case(343)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^3_2 d^5_3 d^7_5
        case(365)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^4_2 d^5_4 d^6_5
        case(442)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^4_5 d^5_1 d^6_4
        case(354)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_5 d^5_1 d^6_2
        case(330)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_6 d^4_1 d^6_4
        case(338)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_6 d^5_7 d^6_5 d^7_4
        case(410)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^4_3 d^5_7 d^7_4
        case(391)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^3_2 d^6_7 d^7_3
        case(367)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_6 d^5_4 d^6_7 d^7_5
        case(409)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_4 d^4_1 d^6_2
        case(324)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_7 d^5_2 d^7_1
        case(196)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^3_1 d^5_3 d^6_5
        case(347)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_6 d^5_2 d^6_5
        case(431)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^3_1 d^4_3
        case(32)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_1 d^4_5 d^5_2
        case(316)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^5_2 d^6_7 d^7_5
        case(385)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_7 d^5_3 d^7_5
        case(396)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_3 d^3_1 d^6_2
        case(312)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_1 d^4_2 d^5_4
        case(317)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_7 d^4_3 d^6_4 d^7_6
        case(401)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_4 d^3_2 d^4_1
        case(30)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_4 d^4_7 d^7_1
        case(210)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_7 d^5_2 d^7_5
        case(366)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_1 d^5_6 d^6_2
        case(328)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_6 d^6_7 d^7_2
        case(369)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^4_1 d^5_7 d^7_4
        case(230)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^4_5 d^5_1
        case(315)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^4_2 d^6_7 d^7_4
        case(379)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_4 d^4_7 d^7_2
        case(357)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_5 d^5_7 d^7_2
        case(363)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^4_7 d^7_1
        case(192)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_6 d^4_3 d^6_2
        case(429)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^5_7 d^6_3 d^7_6
        case(408)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^5_6 d^6_2 d^7_5
        case(388)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_7 d^5_1 d^7_5
        case(215)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_7 d^4_2 d^7_1
        case(190)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^2_7 d^6_2 d^7_1
        case(202)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_6 d^5_7 d^6_5 d^7_3
        case(404)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_7 d^7_1
        case(56)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^2_1 d^3_2 d^7_3
        case(59)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(1,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^4_5 d^5_2 d^6_4
        case(440)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_5 d^5_1
        case(303)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_5 d^4_3 d^5_2
        case(423)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^2_6 d^4_2 d^6_1
        case(319)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^4_5 d^5_1 d^7_4
        case(229)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_6 d^5_3 d^6_2
        case(435)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_6 d^4_1 d^6_4
        case(320)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_7 d^4_3 d^7_2
        case(356)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_4 d^4_2 d^5_3
        case(422)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^4_6 d^5_2 d^6_5
        case(437)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^2_4 d^4_1 d^7_2
        case(193)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^4_6 d^5_4 d^6_3
        case(447)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_6 d^5_3 d^6_5
        case(443)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_7 d^4_1 d^5_4 d^7_5
        case(225)
          csTmp(-1,bin_color) = csIn(1,-1)
          csTmp(0,bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^5_7 d^6_5 d^7_2
        case(386)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^3_2 d^4_3 d^7_4
        case(359)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_1 d^5_6 d^6_3
        case(346)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_4 d^4_5 d^5_6 d^6_3
        case(444)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^5_7 d^6_1 d^7_6
        case(239)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_7 d^6_1 d^7_6
        case(221)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_4 d^4_5 d^5_1
        case(333)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_6 d^3_4 d^4_2 d^6_3
        case(428)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(6,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_7 d^4_5 d^5_2 d^7_4
        case(376)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_6 d^6_7 d^7_1
        case(234)
          csTmp(-1,bin_color) = csIn(6,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^4_1 d^5_6 d^6_4
        case(352)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_7 d^6_1 d^7_6
        case(203)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_6 d^3_7 d^6_3 d^7_1
        case(220)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_5 d^3_2 d^4_3 d^5_4
        case(424)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^2_4 d^4_1 d^5_2
        case(318)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_4 d^4_6 d^6_2
        case(426)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^3_5 d^4_3 d^5_6 d^6_4
        case(445)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^3_7 d^4_1 d^7_4
        case(209)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(4,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_1 d^4_3 d^5_4
        case(335)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^4_6 d^5_1 d^6_5
        case(350)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          csTmp(csIn(6,-1), bin_color) = csIn(5,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_5 d^3_2 d^5_1
        case(301)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_5 d^4_3 d^5_1
        case(331)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(5,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_4 d^4_6 d^6_1
        case(321)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(6,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_3 d^3_4 d^4_1
        case(35)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_4 d^4_5 d^5_2
        case(420)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(5,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_4 d^3_2 d^4_7 d^7_3
        case(355)
          csTmp(-1,bin_color) = csIn(4,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(2,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_7 d^4_3 d^7_1
        case(208)
          csTmp(-1,bin_color) = csIn(3,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_2 d^2_5 d^4_1 d^5_4
        case(314)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(2,0)
          csTmp(csIn(2,-1), bin_color) = csIn(5,0)
          csTmp(csIn(4,-1), bin_color) = csIn(1,0)
          csTmp(csIn(5,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_4 d^3_6 d^4_3 d^6_1
        case(337)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(1,-1), bin_color) = csIn(4,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(3,0)
          csTmp(csIn(6,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^2_3 d^3_6 d^4_2 d^6_4
        case(425)
          csTmp(-1,bin_color) = 0
          csTmp(0,bin_color) = 0
          csTmp(csIn(2,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(2,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_3 d^2_7 d^3_2 d^7_1
        case(58)
          csTmp(-1,bin_color) = csIn(2,-1)
          csTmp(0,bin_color) = csIn(1,0)
          csTmp(csIn(1,-1), bin_color) = csIn(3,0)
          csTmp(csIn(3,-1), bin_color) = csIn(2,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^1_5 d^3_1 d^5_7 d^7_3
        case(218)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(3,0)
          csTmp(csIn(1,-1), bin_color) = csIn(5,0)
          csTmp(csIn(3,-1), bin_color) = csIn(1,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
!       d^4_5 d^5_7 d^6_4 d^7_6
        case(414)
          csTmp(-1,bin_color) = csIn(5,-1)
          csTmp(0,bin_color) = csIn(6,0)
          csTmp(csIn(4,-1), bin_color) = csIn(5,0)
          csTmp(csIn(6,-1), bin_color) = csIn(4,0)
          if (last) then
            csTmp(csLast(-1), bin_color) = csTmp(0, bin_color)
            csTmp(csTmp(-1, bin_color), bin_color) = csLast(0)
            csTmp(-1,bin_color) = 0
            csTmp(0,bin_color) = 0
          end if 
          select_branch(bin_color) = cf + 1
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation7')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation7

    subroutine compute_colour_correlation8(nbranches, colorflowIds, csIn,      &
    U1GluonIn, csOut, U1GluonOut, csLast, select_branch, last, pr, lp, legsE,  &
    ncpower)
      integer,                       intent(in)    :: nbranches
      integer, dimension(nbranches), intent(inout) :: select_branch
      integer, dimension(:,-1:),     intent(in)    :: csIn
      integer, allocatable,          intent(inout) :: csOut(:,:)
      logical, dimension(:),         intent(in)    :: U1GluonIn
      logical,                       intent(inout) :: U1GluonOut
      integer, dimension(-1:0),      intent(in)    :: csLast
      logical,                       intent(in)    :: last
      integer,                       intent(in)    :: pr, legsE, lp
      integer, dimension(0:),        intent(in)    :: colorflowIds
      integer, dimension(nbranches), intent(inout) :: ncpower

      integer, dimension(-1:legsE,nbranches) :: csTmp
      integer :: bin_color  ! Number of valid colorflows
      integer :: cf
      integer :: i, j, ib
      bin_color = 1
      cf = 0
      do while(cf < size(colorflowIds))
        ! select_branch is zero if it has been filtered before, and the
        ! corresponding colorflow is not computed
        if (select_branch(cf+1) == 0) then
          cf = cf+1
          cycle
        end if
        csTmp(-1:legsE, bin_color) = 0 ! Init as colorless
        do i = 1, legsE
          do j = 1, size(csIn,1)
            ! Copying color deltas
            csTmp(i, bin_color) = csTmp(i, bin_color) + csIn(j,i)
          end do
        end do
        select case(colorflowIds(cf))
        case default
          call error_mdl('Unknown colourflow id.', where                       &
          ='compute_colour_correlation8')
        end select 
        cf = cf +1
        bin_color = bin_color + 1
      end do 
      bin_color = bin_color - 1
      if (last .and. (lp .eq.1)) then
        do ib = 1, bin_color
          if (csTmp(legsE - 1, ib).eq.legsE) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE) then
                csTmp(j, ib) = csTmp(legsE - 1, ib)
              end if
            end do
          end if
          csTmp(legsE - 1, ib) = 0
          if (csTmp(legsE, ib) .eq. legsE - 1) then
            ncpower(ib) = ncpower(ib) + 1
          else
            do j = 1, legsE
              if (csTmp(j, ib).eq.legsE - 1) then
                csTmp(j, ib) = csTmp(legsE, ib)
              end if
            end do
          end if
          csTmp(legsE, ib) = 0
        end do
      end if
      ! Outgoing colorflow is adjusted to the actual number of valid branches
      allocate(csOut(-1:legsE, bin_color))
      do ib = 1, bin_color
        csOut(-1:legsE, ib) = csTmp(-1:legsE, ib)
      end do
      ! select_branch has been updated and valid branches are filled gapless.
      ! non-valid branches are set to zero.
      if (bin_color .lt. size(select_branch)) then
        select_branch(bin_color+1:size(select_branch)) = 0
      end if
    end subroutine compute_colour_correlation8

end module class_colourflow
